
#include "HDC1000_Defs.h"

extern sbit sRDY;
char tmp_data[12];

static unsigned int temp_value;
static unsigned int humidity_value;
float dewpoint;
float hdc1000_temp;
float hdc1000_humidity;

void HDC1000_Setup() {
  tmp_data[0] = 0x02;
  tmp_data[1] = 0x10;
  tmp_data[2] = 0x00;
  I2C2_Start();
  I2C2_Write(0x40,tmp_data,3,END_MODE_STOP);
  delay_ms(5);
}

//Conversion time for 14bit values of temperature and humidity 6,5ms + 6,35ms
static void wait_conversion() {
  delay_ms(13);
}

static void Float_Conversion() {
  hdc1000_temp = ((float)temp_value * Tconstant)-40;
  hdc1000_humidity = (float)humidity_value * Hconstant;
}

static void Read_Sensor() {
  tmp_data[0] = TEMP_I2C_ADDR;
  I2C2_Start();
  I2C2_Write(0x40,tmp_data,1,END_MODE_RESTART);
  wait_conversion();
  I2C2_Read(HDC1000_I2C_ADDR,tmp_data,4,END_MODE_STOP);
  temp_value = (tmp_data[0] << 8) + tmp_data[1];
  humidity_value = (tmp_data[2] << 8) + tmp_data[3];
}

void HDC1000_Read() {
  Read_Sensor();
  Float_Conversion();
}


unsigned int HDC1000_DeviceID() {
  unsigned int id_value;
  tmp_data[0] = DEVICEID_I2C_ADDR;
  I2C2_Start();
  I2C2_Write(0x40,tmp_data,1,END_MODE_RESTART);
  I2C2_Read(HDC1000_I2C_ADDR,tmp_data,2,END_MODE_STOP);
  id_value = (tmp_data[0] << 8) + tmp_data[1];
  return id_value;
}

//August-Roche-Magnus approximation.
//http://en.wikipedia.org/wiki/Clausius%E2%80%93Clapeyron_relation#Meteorology_and_climatology
//tested: http://andrew.rsmas.miami.edu/bmcnoldy/Humidity.html
//RH: =100*(EXP((17.625*TD)/(243.04+TD))/EXP((17.625*T)/(243.04+T)))
//TD: =243.04*(LN(RH/100)+((17.625*T)/(243.04+T)))/(17.625-LN(RH/100)-((17.625*T)/(243.04+T)))
//T: =243.04*(((17.625*TD)/(243.04+TD))-LN(RH/100))/(17.625+LN(RH/100)-((17.625*TD)/(243.04+TD)))
//output value in celsius
void Calculate_Dewpoint() {
  dewpoint = 243.04*(log(hdc1000_humidity/100)+((17.625*hdc1000_temp)/(243.04+hdc1000_temp)))
            /(17.625-log(hdc1000_humidity/100)-((17.625*hdc1000_temp)/(243.04+hdc1000_temp)));
}