/*******************************************************************************
 * Project name:
     MiniWebServer example
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20150507:
       - initial release (DR);
 * Test configuration:
     MCU:             STM32F407ZG
                      http://www.st.com/st-web-ui/static/active/en/resource/technical/document/datasheet/DM00037051.pdf
     Oscillator:      HS-PLL 150.0000 MHz, 25.0000 MHz Crystal
     ext. modules:    mikromedia + for stm shield : 
                      http://www.mikroe.com/mikromedia/plus/shields/stm32/
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
*******************************************************************************/

#include "MiniWebServer.h"


//******************************************************************************
//                                  TIMER
//******************************************************************************
void Timer2_interrupt() iv IVT_INT_TIM2 {
  TIM2_SR.UIF = 0;
  Net_Ethernet_Intern_UserTimerSec++ ;                                          //Increment ethernet library counter
  timeout_cnt++;
}

void InitTimer2(){
  RCC_APB1ENR.TIM2EN = 1;
  TIM2_CR1.CEN = 0;
  TIM2_PSC = 1124;
  TIM2_ARR = 63999;
  NVIC_IntEnable(IVT_INT_TIM2);
  TIM2_DIER.UIE = 1;
}

void Ethernet_Msg(char *text){
  TFT_Rectangle(10,10,480,29);
  TFT_Write_Text(text,10,10);
}




void ConvertTimestampToUnix(char *TimestampStr) {
   long multiplier;
   char n;
   char *ptr1;
   multiplier = 1;
   ptr1 = TimestampStr;
   while(*ptr1 != 0) {
      n = *ptr1;
      n -= 48;
      UnixTime += (long)n * multiplier;
      multiplier *= 10;
      ptr1++;
   }
}


//api string json example: 
//{"status":"OK","message":"","countryCode":"RS","zoneName":"Europe\/Belgrade",
//"abbreviation":"CEST","gmtOffset":"7200","dst":"1","timestamp":1429560893}

char GetTimestampFromJson() {
  char *ptr1, *ptr2;
  char timestamp[14];
  ptr1 = strstr(getRequest,"}");                //we search json eof
  if (!ptr1) {
    return 0;
  }
  ptr2 = timestamp;
  ptr1--;                              //remove '}' character
  while (*ptr1) {                      //we get reversed string of timestamp
     if(*ptr1 == ':') {                //we found start of unix Time string
        break;
     }
     *ptr2++ = *ptr1--;
  }
  *ptr2 = 0;
  TFT_Write_Text(timestamp,10,50);
  ConvertTimestampToUnix(timestamp);
  LongToStr(UnixTime,text);
  TFT_Write_Text(text,10,70);
  return 1;
}

void mySocketDisconnect() {
   char n;
   Ethernet_Msg("Disconnecting...");
   while (n!=1) {
      n = Net_Ethernet_Intern_disconnectTCP(mySocket);
   }
}

//******************************************************************************
//                            DDNS SYNC
//******************************************************************************

char RenewNoIP() {
  Net_Ethernet_Intern_putStringTCP("GET /nic/update?hostname=your.ddns.net HTTP/1.1\r\n", mySocket);                               //your.ddns.net
  Net_Ethernet_Intern_putStringTCP("Host: dynupdate.noip.com\r\n", mySocket);
  Net_Ethernet_Intern_putStringTCP("User-Agent: mikro Web Server/v1.0 yourname@mail.com\r\n", mySocket);
  Net_Ethernet_Intern_putStringTCP("Authorization: Basic bmFtZTpwYXNzd29yZA==\r\n", mySocket);                             //Authorization
  Net_Ethernet_Intern_putStringTCP("Connection: close\r\n", mySocket);                                                             //'name:password' converted in base64 format
  Net_Ethernet_Intern_putStringTCP("Accept: text/html\r\n", mySocket);
  Net_Ethernet_Intern_putStringTCP("Accept-Language: en-US\r\n", mySocket);
  Net_Ethernet_Intern_putStringTCP("\r\n\r\n", mySocket);
  return 1;
}

void NoIp_ConnectTCP() {
  char i;
  Ethernet_Msg("Connecting to no-ip...");
  while(i!=1) {
    //Open connection with remote host.
    i = Net_Ethernet_Intern_connectTCP(NoIp_Addr, 80, 8080, &mySocket);
  }
}

void DDNS_Sync() {
  Status = _WEB_SERVER_SYNC_DDNS;
  NoIp_ConnectTCP();
  Ethernet_Msg("Waiting connection to dynupdate.no-ip.com server...");
  NoIp_mark = 0;
  NoIp_Renew_mark = 0;
  NoIp_Response_mark = 0;
  reqTimeOut = Net_Ethernet_Intern_userTimerSec + 20;
  while(NoIp_Response_mark == 0) {                                              //wait to renew my public IP on noip server
   Net_Ethernet_Intern_doPacket();
   if (Net_Ethernet_Intern_userTimerSec > reqTimeOut) {
      Net_Ethernet_Intern_closeSocketTCP(mySocket);
      NoIp_ConnectTCP();
      NoIp_mark = 0;
      NoIp_Renew_mark = 0;
      reqTimeOut = Net_Ethernet_Intern_userTimerSec + 10;
    }
    if((mySocket->state == 3) && (NoIp_mark == 0)) {
      Net_Ethernet_Intern_startSendTCP(mySocket);
      NoIp_mark = 1;
    }
  }
  MySocketDisconnect();                                 //close socket with no-ip server
  Ethernet_Msg("no-ip udated");
  delay_ms(2000);
}


//******************************************************************************
//                            TIME SYNC
//******************************************************************************
void TimeZoneServer_ConnectTCP() {
  char n;
  Ethernet_Msg("Connecting to Time server...");
    while(n!=1) {
    //Open connection with remote host.
    n = Net_Ethernet_Intern_connectTCP(TimeZone_Addr, 80, 8080, &mySocket);
  }
}


//Time API from timezonedb.com
char GetTimeFromServer() {
  Net_Ethernet_Intern_putStringTCP("GET /?zone=Europe/Belgrade&key=yourkey&format=json HTTP/1.1\r\n", mySocket);       //KEY !!!
  Net_Ethernet_Intern_putStringTCP("Host: api.timezonedb.com\r\n", mySocket);
  Net_Ethernet_Intern_putStringTCP("User-Agent: mikro Web Server/v1.0 your@mail.com\r\n", mySocket);                  //mail !!!
  Net_Ethernet_Intern_putStringTCP("Connection: close\r\n", mySocket);
  Net_Ethernet_Intern_putStringTCP("Accept: text/html\r\n", mySocket);
  Net_Ethernet_Intern_putStringTCP("Accept-Language: en-US\r\n", mySocket);
  Net_Ethernet_Intern_putConstStringTCP("\r\n\r\n", mySocket);
  return 1;
}


void TIME_SYNC() {
  Status = _WEB_SERVER_SYNC_TIME;
  TimeZoneServer_ConnectTCP();
  Ethernet_Msg("Waiting response from Time server...");
  TimeZone_mark = 0;
  TimeZone_Request_mark = 0;
  TimeZone_Response_mark = 0;
  reqTimeOut = Net_Ethernet_Intern_userTimerSec + 10;
  while(TimeZone_Response_mark == 0) {
   Net_Ethernet_Intern_doPacket();
   if (Net_Ethernet_Intern_userTimerSec > reqTimeOut) {
      Net_Ethernet_Intern_closeSocketTCP(mySocket);
      TimeZoneServer_ConnectTCP();
      TimeZone_mark = 0;
      TimeZone_Request_mark = 0;
      reqTimeOut = Net_Ethernet_Intern_userTimerSec + 10;
    }
    if((mySocket->state == 3) && (TimeZone_mark == 0)) {
      Net_Ethernet_Intern_startSendTCP(mySocket);
      TimeZone_mark = 1;
    }
  }
  //Parse time from timeserver request content
  GetTimestampFromJson();
  MySocketDisconnect();                                                         //close socket from timezone server
  Ethernet_Msg("Time udated");
  delay_ms(2000);
}



//******************************************************************************
//                          MIKRO WEB SERVER
//******************************************************************************



void StartPageSending(SOCKET_Intern_Dsc *socket) {
  pageHandler.pageSocket = socket;
  pageHandler.sendState = _PAGE_HANDLER_STATE_SENDING;
  pageHandler.currDataPtr = &pageBuffer;
  pageHandler.endDataPtr = &pageBuffer + pageLength;
}

// Processing TCP data.
void Net_Ethernet_Intern_UserTCP(SOCKET_Intern_Dsc *socket){
  unsigned long restToSend;
  unsigned int cnt;
  
  if(socket->destPort != 8080) {                                                 //I listen only 8080 port
    return;
  }
  mySocket = socket;
  switch (status) {
    case _WEB_SERVER_SYNC_DDNS:
       //***********************************************************************
       //                 Response from dynupdate.noip.com
       //***********************************************************************
       if((memcmp(socket->remoteIP,NoIp_Addr,4) == 0) ){
          if (socket->dataLength >= 15) {
             for(cnt=0; cnt < 15; cnt++) {
                getRequest[cnt]= Net_Ethernet_Intern_GetByte();
             }
             if(strstr(getRequest,"OK")){                                       //wait to http response HTTP/1.1 200 OK or HTTP/1.0 200 OK
               NoIp_Response_mark = 1;
               NoIp_mark = 1;
               NoIp_Renew_mark = 1;
               mySocket = socket;                                               //point current socket to mySocket
             }

          }
         if(NoIp_Renew_mark == 0) {
            Ethernet_Msg("Sending HTTP to dynupdate.noip.com");
            NoIp_Renew_mark = RenewNoIP();
         }

       }
       break;
    case _WEB_SERVER_SYNC_TIME:
       //***********************************************************************
       //            Response from api.timezonedb.com
       //***********************************************************************
          if (socket->dataLength >= 15) {
             for(cnt=0; cnt < socket->dataLength; cnt++) {
                getRequest[cnt]= Net_Ethernet_Intern_GetByte();
             }
             getRequest[cnt] = 0;
             Ethernet_Msg(getRequest);
             if(strstr(getRequest,"OK")){                                       //wait to http response HTTP/1.1 200 OK or HTTP/1.0 200 OK
               TimeZone_Response_mark = 1;
               TimeZone_mark = 1;
               TimeZone_Request_mark = 1;
               mySocket = socket;                                               //point current socket to mySocket
             }

          }
         if(TimeZone_Request_mark == 0) {
            Ethernet_Msg("Sending HTTP to api.timezonedb.com");
            TimeZone_Request_mark = GetTimeFromServer();
         }
       break;
    case _WEB_SERVER_ACTIVE:
       //***********************************************************************
       //                           WEB SERVER
       //***********************************************************************
       if (socket->dataLength >= 16) {
          for(cnt=0; cnt < 16; cnt++) {
             getRequest[cnt]= Net_Ethernet_Intern_GetByte();
          }
          //Ethernet_Msg(getRequest);
          if (strstr(getRequest, httpMethod)) {
              switch (getRequest[5]) {
                 case 'p':
                     switch (getRequest[6]) { //select page
                         case '0':
                            visit_cnt++;
                            Create_IndexPage();
                            StartPageSending(socket);
                         case '1':
                            Create_Page1();
                            StartPageSending(socket);
                            break;
                         case '2':
                            Create_Page2();
                            StartPageSending(socket);
                            break;
                     }
                     break;
                 case 't':
                     if (strstr(getRequest,"R1")) {
                         RELAY_1 = ~RELAY_1; //toggle RELAY 1
                     }
                     if (strstr(getRequest,"R2")) {
                         RELAY_2 = ~RELAY_2; //toggle RELAY 2
                     }
                     Create_Page2();
                     StartPageSending(socket);
                     break;
                 default:
                     visit_cnt++;
                     Create_IndexPage();
                     StartPageSending(socket);
              }

          }
       }

       if((pageHandler.sendState == _PAGE_HANDLER_STATE_SENDING) && (pageHandler.pageSocket == socket) ) {

          // Put data
          restToSend = pageHandler.endDataPtr - pageHandler.currDataPtr;
          pageHandler.currDataPtr += Net_Ethernet_Intern_putBytesTCP(pageHandler.currDataPtr, restToSend, socket);

          // Check if all data sent
          if(pageHandler.currDataPtr == pageHandler.endDataPtr) {
              pageHandler.sendState = _PAGE_HANDLER_STATE_FINISHED;
          }

       }
  }

}

// Processing UDP data.
unsigned int Net_Ethernet_Intern_UserUDP(UDP_Intern_Dsc *udpDsc) {
  return 0;
}

void CloseFinishedSocket() {

    if((pageHandler.sendState == _PAGE_HANDLER_STATE_FINISHED)       // If all data sent
       && (pageHandler.pageSocket->state == _SOCKET_INTERN_DSC_STATE_CONN_ESTABLISHED) // If the connection is still established
       && (Net_Ethernet_Intern_bufferEmptyTCP(pageHandler.pageSocket)))   {        // If there is no non-acknowledged data


        Net_Ethernet_Intern_disconnectTCP(pageHandler.pageSocket);           // Disconnect
        pageHandler.sendState = _PAGE_HANDLER_STATE_IDLE;                    // Go to idle state
        pageHandler.pageSocket = 0;                                          // Reset pointer to socket

    }

}

//******************************************************************************
//                            END MIKRO WEB SERVER
//******************************************************************************



static char Init_HDC1000() {
  GPIO_Set_Pin_Mode(&GPIOE_BASE, _GPIO_PIN_3, _GPIO_CFG_MODE_INPUT | _GPIO_CFG_PULL_UP); //PE3 - Data Ready pin
  GPIO_Digital_Input (&GPIOF_BASE, _GPIO_PINMASK_0);  //SDA/PF0
  GPIO_Digital_Output (&GPIOF_BASE, _GPIO_PINMASK_1); //SCL/PF1  R2
  //I2C INITIALIZATION
  //If bus is busy wait SDA high before initializing I2C module
  GPIOF_ODR.B1 = 1;
  while (GPIOF_IDR.B0 == 0) {
    GPIOF_ODR.B1 = 0;
    Delay_us(10);
    GPIOF_ODR.B1 = 1;
    Delay_us(10);
  }
  I2C2_Init_Advanced(400000, &_GPIO_MODULE_I2C2_PF01);
  delay_ms(200);
  HDC1000_Setup();
  return 1;
}

static void Init_MCU() {
  GPIO_Digital_Output (&GPIOE_BASE, _GPIO_PINMASK_6 | _GPIO_PINMASK_4);         //RELAY2 & MOTION EN
  GPIO_Digital_Output (&GPIOD_BASE, _GPIO_PINMASK_15);                          //RELAY1
  GPIO_Digital_Input (&GPIOE_BASE, _GPIO_PINMASK_1);                            //MOTION OUT
  // Select Analog channel
  GPIO_Analog_Input(&GPIOF_BASE,  _GPIO_PINMASK_9);                             //Light intensity sensor on mikromedia+
  GPIO_Analog_Input(&GPIOB_BASE, _GPIO_PINMASK_1);                               //AIRQ click ADC12_IN9
  ADC3_Init();
  ADC1_Init();
  Init_HDC1000();
  InitTimer2();
}


void main() {
  Init_TFT();
  Init_Display(480,272);
  //Init_TouchPanel();                                                          //Not use for this example
  TFT_Set_Pen(CL_BLUE, 0);
  TFT_Set_Font(TFT_defaultFont, CL_WHITE, FO_HORIZONTAL);
  TFT_Set_Brush(1,CL_BLUE,0,0,0,0);
  delay_ms(100);
  Init_MCU();
  TFT_Fill_Screen(CL_BLUE);
  Status = _WEB_SERVER_STOPPED;
  Ethernet_Msg("Ethernet Initialization...");
  //Ethernet
  Net_Ethernet_Intern_stackInitTCP();
  Net_Ethernet_Intern_Init(myMacAddr, myIpAddr, _ETHERNET_FULLDUPLEX, &_GPIO_MODULE_ETHERNET_G11_13_14);
  Net_Ethernet_Intern_confNetwork(ipMask, gwIpAddr, dnsIpAddr);
  while(!(regValue & 0b100)) {                                                  // Wait for active link
    Net_Ethernet_Intern_PHY_Read(_PHY_BSR, &regValue);
  }
  TIM2_CR1.CEN = 1;                                                             //Start timer
  delay_ms(100);
  //DDNS_Sync();                                                                //Update No-IP service. You must create free account on noip.com
  //TIME_Sync();                                                                //Update Time from Time server. RTCC not yet implemented
  Status = _WEB_SERVER_ACTIVE;
  Ethernet_Msg("Mikro Web Server started.");
  TFT_Write_Text("test",10,110);
  MOTION_EN = 1;
  motion_old = MOTION_OUT;
  motion_cnt = 0;
  Read_Sensors();
  timeout_cnt = 0;
  TFT_Image_Jpeg(0, 0,iot_image_red1_JPG);
  while (1) {
    Net_Ethernet_Intern_doPacket();
    CloseFinishedSocket();
    if((motion_old == 0) && (MOTION_OUT == 1)) {
      motion_cnt++;
    }
    motion_old = MOTION_OUT;
    if (timeout_cnt >= TIMEOUT) {
       Read_Sensors();                                                          //Read all sensors
       timeout_cnt = 0;                                                         //reset counter
    }
    
  }

}