
#include "HDC1000_Driver.h"
//#include "HDC1000_Defs.h"

//sensors
static const VREF = 2048 ;
static const float Vadc_33 = 0.00081;                                                 // ADC step 3,3V/4095 0,81mV (12bit ADC)
static const float Vadc_18 = 0.00044;                                                 // ADC step 1,8V/4095 0,44mV (12bit ADC)
static float lgPPM, ratio, Vrl, sRs;
float light_intensity;
//******************     Air quality click    **********************************
const float aq_Rl      = 10000.0;                // Rl (Ohm) - Load resistance
float airq_ppm;                                  // ppm output result
//******************************************************************************

//******************     CO click    *******************************************
static const float co_Rl      = 10000.0;       // Rl (Ohm) - Load resistance
float co_ppm;                                  // ppm output result
//******************************************************************************

//Calculation of PPM for Air quality click
static float Get_AirQuality() {
   unsigned int air;
   float result;
   air = ADC1_Get_Sample(9);
   Vrl = (float)air * Vadc_33;                                                   // For 5V Vcc use Vadc_5  and  for 3V Vcc use Vadc_33
   sRs = aq_Rl * (5 - Vrl)/Vrl;                                                  // Calculate sensor resistance
   ratio = sRs/aq_Rl;                                                            // Calculate ratio
   lgPPM = (log10(ratio) * -0.8)+ 0.9;                                           // Calculate ppm
   result = pow(10,lgPPM);
   return result;
}

//Calculation of PPM for CO click
static float Get_CO() {
   unsigned int cov;
   float result;
   cov = ADC1_Get_Sample(8);
   Vrl = (double)cov * Vadc_33;                                                  // For 5V Vcc use Vadc_5  and  for 3V Vcc use Vadc_33
   sRs = co_Rl * (5 - Vrl)/Vrl;                                                  // Calculate sensor resistance
   ratio = sRs/co_Rl;                                                            // Calculate ratio
   lgPPM = (log10(ratio) * -0.8)+ 0.9;                                           // Calculate ppm
   result = pow(10,lgPPM);
   return result;
}

static float Get_Light_Intensity(){
  unsigned int light_value;
  float result;
  light_value = ADC3_Get_Sample(7);
  delay_ms(10);
  light_value = light_value >> 2;
  if (light_value > 1000) {
    light_value = 1000;
  }
  result = (float)light_value * 0.1;
  return result;
}

void Read_Sensors() {
  HDC1000_Read();
  light_intensity = Get_Light_Intensity();
  airq_ppm = Get_AirQuality();
  //co_ppm = Get_CO();
  Calculate_Dewpoint();
}